/**

  The Initial Developer of the Original Code is
  Matthieu  - http://www.programmation-facile.com/
  Portions created by the Initial Developer are Copyright (C) 2013
  the Initial Developer. All Rights Reserved.

  Contributor(s) :

 */



/**
 * Le Framework de jeux JavaScript Phaser
 *
Phaser v2.0.5 - http://phaser.io - @photonstorm - (c) 2014 Photon Storm Ltd. 
https://github.com/photonstorm/phaser
http://examples.phaser.io/
 */



/**
 * Les variables du jeu
 * 
 */

var game; 


var tileSize = 50;// taille d'une case
var nSizeColLign = 6;// nombre de lignes et colonnes
var nNbreTileTypes = 6;// nombre de cases différentes

var bDragPossible = false;// déplacement possible
var movingRow;// ligne déplacée
var movingCol;// colonne déplacé

var tileArray = [];// contient toutes les cases

var startX;// coordonnées sauvegardées lorsque le joueur commence le déplacement
var startY;// coordonnées sauvegardées lorsque le joueur commence le déplacement

var distX;// distance de déplacement horizontal
var distY;// distance de déplacement vertical

var dragDirection = "";// direction du déplacement
var tempTile;// case temporaire

// les dimensions du jeu
var nWidth = 300;
var nHeight = 300;


document.addEventListener("DOMContentLoaded", Main, false); // appel au chargement de la page


/**
 * Fonction principale
 * Appelée au chargement de la page
 * 
 */
function Main()
{
    console.log("Main");

    // creation de la zone de jeu - API canvas
    game = new Phaser.Game(nWidth, nHeight, Phaser.AUTO, "idGameDiv", {preload:onPreload, create:onCreate, update:onUpdate});
}


/**
 * Préchargement des éléments du jeu
 * 
 * @return {[type]} [description]
 */
function onPreload()
{ 
    game.stage.backgroundColor = '#cccccc';// la couleur de fond du jeu
    game.load.spritesheet("tiles", "assets/cases.png", tileSize, tileSize);// charge des sprites graphiques
}


/**
 * Génère un nombre entier aléatoire entre min (inclus) et max (inclus)
 * 
 */
function getRandomInt(min, max)
{
    return Math.floor( Math.random() * (max - min + 1) ) + min;
}




/**
 * Création de la scène du jeu,
 * mise en place des graphismes...
 * 
 * @return {[type]} [description]
 */
function onCreate() 
{
    // affiche le jeu en plein écran
    /*game.scale.scaleMode = Phaser.ScaleManager.SHOW_ALL;
    game.scale.setScreenSize();*/

    // création de la carte du jeu avec les cases
    for(i=0;i<nSizeColLign;i++)
    {
        tileArray[i]=[];

        for(j=0;j<nSizeColLign;j++)
        {
            var randomTile = getRandomInt(0, 6);// sort une case de couleur aléatoire

            theTile = game.add.sprite(j*tileSize,i*tileSize,"tiles");
            
            // ajout des propriétés "frame" and "value"
            theTile.frame = randomTile;
            theTile.value = randomTile;
            tileArray[i][j]=theTile;    
        }
    }

    // ajout d'une case temporaire et invisible
    tempTile = game.add.sprite(0,0,"tiles");
    tempTile.visible = false;

    // écouteur du clic de la souris
    game.input.onDown.add(clicCase, this);
}


/**
 * Appelée en continu pendant le jeu
 * 
 * @return {[type]} [description]
 */
function onUpdate() 
{
    if(bDragPossible)
    {
        distX = game.input.worldX-startX;
        distY = game.input.worldY-startY;

        switch(dragDirection)
        {
            // choix de la direction du déplacement
            case "":
                var dist = distX*distX+distY*distY;

                if (dist>25)
                {
                    var dragAngle=Math.abs(Math.atan2(distY,distX));

                    if ((dragAngle>Math.PI/4 && dragAngle<3*Math.PI/4))
                        dragDirection="vertical";
                    else 
                        dragDirection="horizontal";
                }
            break;

            // déplacement horizontal
            case "horizontal":
                tempTile.visible = false;
                tempTile.y=movingRow*tileSize;

                var deltaX = (Math.floor(distX/tileSize)%nSizeColLign);

                if (deltaX>=0)
                    tempTile.frame = tileArray[movingRow][nSizeColLign-1-deltaX].value; 
                else
                {
                    deltaX=deltaX*-1-1;
                    tempTile.frame = tileArray[movingRow][deltaX].value;        
                }

                for(i=0;i<nSizeColLign;i++)
                {
                    tileArray[movingRow][i].x = (i*tileSize+distX)%(tileSize*nSizeColLign);

                    if (tileArray[movingRow][i].x<0) 
                        tileArray[movingRow][i].x+=tileSize*nSizeColLign;

                    if(distX%tileSize>0)
                    {
                        tempTile.visible = true;
                        tempTile.x=distX%tileSize-tileSize;
                    }

                    if(distX%tileSize<0)
                    {
                        tempTile.visible = true;
                        tempTile.x=distX%tileSize;
                    } 
                }
            break;

            // déplacement vertical
            case "vertical":
                tempTile.visible = false;
                tempTile.x=movingCol*tileSize;

                var deltaY = (Math.floor(distY/tileSize)%nSizeColLign);

                if (deltaY>=0)                                    
                    tempTile.frame = tileArray[nSizeColLign-1-deltaY][movingCol].value;
                else
                {
                    deltaY=deltaY*-1-1;
                    tempTile.frame = tileArray[deltaY][movingCol].value;        
                }

                for(i=0;i<nSizeColLign;i++)
                {
                    tileArray[i][movingCol].y=(i*tileSize+distY)%(tileSize*nSizeColLign);
                   
                    if (tileArray[i][movingCol].y<0) 
                        tileArray[i][movingCol].y+=tileSize*nSizeColLign;
                    
                    if(distY%tileSize>0)
                    {
                        tempTile.visible = true;
                        tempTile.y=distY%tileSize-tileSize;
                    }
                    
                    if(distY%tileSize<0)
                    {
                        tempTile.visible = true;
                        tempTile.y=distY%tileSize;
                    }   
                }
            break;
        }
    }
}


/**
 * une case a été cliqué par le joueur
 * 
 * @return {[type]} [description]
 */
function clicCase()
{
    // sauvegarde des coordonnées
    startX = game.input.worldX;
    startY = game.input.worldY;

    // sauvegarde des colonnes et lignes concernées
    movingRow = Math.floor(startY/tileSize);
    movingCol = Math.floor(startX/tileSize);

    bDragPossible = true;// déplacement autorisé

    // mise à jour des écouteurs de la souris
    game.input.onDown.remove(clicCase, this);// clic
    game.input.onUp.add(releaseTile, this);// relachement du clic
}


/**
 * Relachement de la souris
 * Arrêt du déplacement des cases concernées
 * 
 * @return {[type]} [description]
 */
function releaseTile()
{
    // did we drag horizontally or vertically?
    switch(dragDirection)
    {
        case "horizontal":

                var shiftAmount = Math.floor(distX/(tileSize/2));
                shiftAmount = Math.ceil(shiftAmount/2)%nSizeColLign;
                var tempArray = [];
               
                if(shiftAmount>0)
                {
                    for(i=0;i<nSizeColLign;i++)
                    {
                        tempArray[(shiftAmount+i)%nSizeColLign] = tileArray[movingRow][i].value;
                    }
                }
                else
                {
                    shiftAmount *= -1;

                    for(i=0;i<nSizeColLign;i++)
                    {
                        tempArray[i] = tileArray[movingRow][(shiftAmount+i)%nSizeColLign].value;
                    }
                }

                for(i=0;i<nSizeColLign;i++)
                {
                    tileArray[movingRow][i].value = tempArray[i];
                    tileArray[movingRow][i].frame = tempArray[i];
                    tileArray[movingRow][i].x = i*tileSize;
                }
        break;

        case "vertical":

                var shiftAmount = Math.floor(distY/(tileSize/2));
                shiftAmount = Math.ceil(shiftAmount/2)%nSizeColLign;
                var tempArray = [];
               
                if(shiftAmount>0)
                {
                    for(i=0;i<nSizeColLign;i++)
                    {
                        tempArray[(shiftAmount+i)%nSizeColLign] = tileArray[i][movingCol].value;
                    }
                }
                else
                {
                    shiftAmount *= -1;
                    for(i=0;i<nSizeColLign;i++)
                    {
                        tempArray[i]=tileArray[(shiftAmount+i)%nSizeColLign][movingCol].value;
                    }
                }

                for(i=0;i<nSizeColLign;i++)
                {
                    tileArray[i][movingCol].value = tempArray[i];
                    tileArray[i][movingCol].frame = tempArray[i];
                    tileArray[i][movingCol].y=i*tileSize;
                }
        break;  

    }

    // autorise le joueur a choisir une nouvelle ligne ou colonne de cubes à déplacer
    dragDirection = "";
    bDragPossible = false;
    tempTile.visible=false;

    // mise à jour des écouteurs de la souris
    game.input.onUp.remove(releaseTile, this);// relachement
    game.input.onDown.add(clicCase, this);// clic cubes

}





